 <?php
class DocumentController extends ActionController {
	
	public function __construct($viewPath) {
		parent::__construct($viewPath);
	}
	
	public function getlist() {
		$acl = new ACL();
		$docs = Document::GetAll();
		$allowed_docs = array();
		foreach ($docs as $doc) {
			if ($acl->HasPermission($this->current_user, $doc, ACL::READ)) {
				$allowed_docs[] = $doc;
			}
		}
		$this->view->load('document_list', array('allowed_docs'=>$allowed_docs));
		$this->view->render();
	}
	
	public function view($id) {
		$doc = Document::GetById($id);
		$acl = new ACL();
		if ($p = $acl->HasPermission($this->current_user, $doc, ACL::READ)) {
			$this->view->load('document_view', array('doc'=>$doc));
			$this->view->render();
		} else {
			$this->redirect('/error/denied');
		}
	}
	
	public function edit($id) {
		$doc = Document::GetById($id);
		$acl = new ACL();
		
		if (! $acl->HasPermission($this->current_user, $doc, ACL::READ)) {
			$this->redirect('/error/denied');
		}
		
		$required_fields = array('title', 'body');
		if ($this->arrayContains($required_fields, $_POST)) {
			$doc->title = $_POST['title'];
			$doc->content = $_POST['body'];
			$doc->Update();
			$this->redirect('/document/view/'.$doc->id);
		} else {
			$data = array('doc'=>$doc, 'action'=>'edit');
			$this->view->load('document_edit', $data);
			$this->view->render();
		}
	}
	
	public function delete($id) {
		$doc = Document::GetById($id);
		$acl = new ACL();
		
		if (! isset($_POST['confirm'])) {
			$this->view->load('document_delete', array('doc'=> $doc, 'title'=> 'Delete document'));
			$this->view->render();
		} elseif ($acl->HasPermission($this->current_user, $doc, ACL::DELETE)) {
			$doc->Delete();
			$this->redirect('/document/list/');
		} else {
			$this->redirect('/error/denied/');
		}
	}
	
	public function create() {
		$doc = new Document();
		$acl = new ACL();
		
		// Do we need a permission check here?
		$required_fields = array('title', 'body');
		if ($this->arrayContains($required_fields, $_POST)) {
			$doc->title = $_POST['title'];
			$doc->content = $_POST['body'];
			$doc->owner = $this->current_user;
			$doc->insert();
			$acl->SetDefaultPermissions($doc);
			$this->redirect('/document/view/'.$doc->id);
		} else {
			$data = array('doc'=>$doc, 'action'=>'create');
			$this->view->load('document_edit', $data);
			$this->view->render();
		}
	}
	
	public function changeOwner($id) {
		$doc = Document::GetById($id);
		
		if (! $doc) {
			$this->redirect('/error/invalid/');
		}
		
		if (! $this->current_user->isAdministrator()) {
			$this->redirect('/error/denied/');
		}
		
		if ($this->arrayContains(array('username'), $_POST)) {
			$usr = User::GetByName($_POST['username']);
			if (! $usr) {
				$this->redirect('/error/invalid/');
			} else {
				$doc->owner = $usr;
				$ret = $doc->Update();
				if ($ret) $this->redirect('/document/view/'.$id);
				else $this->view->setMessage("Error setting document owner", "error");
			}
		} else {
			$users = User::GetAll();
			$data = array(
				'users'=> $users,
				'doc'=> $doc,
				'title'=> 'Change Document Owner'
			);
			$this->view->load('document_changeowner', $data);
			$this->view->render();
		}
	}
	
	public function changePermissions($id) {
		$db = new DB();
		$doc = Document::GetById($id);
		if (! $doc) {
			$this->redirect('/error/invalid/');
		}
		$user_perms = $db->SelectUserPermissionByDocument($id);
		$group_perms = $db->SelectGroupPermissionByDocument($id);
		$data = array('user_perms'=>  $this->groupPermsByField($user_perms, 'username'),
					  'group_perms'=> $this->groupPermsByField($group_perms, 'group_name'),
					  'doc'=> $doc, 
					  'title'=> 'Change document permissions');
		$this->view->load('document_changepermissions', $data);
		$this->view->render();
	}
	
	protected function groupPermsByField($rowset, $fieldname) {
		$ret = array();
		foreach ($rowset as $row) {
			if (! isset($ret[$row[$fieldname]])) {
				$ret[$row[$fieldname]] = array();
			}
			$ret[$row[$fieldname]][$row['permission']] = $row['granted'];
		}
		return $ret;
	}
	
	public function setUserPermission($id) {
		$doc = Document::GetById($id);
		$acl = new ACL();
		
		if (! $doc) {
			$this->redirect('/error/invalid/');
		}
		
		if (! $this->current_user->isAdministrator() &&
			  $this->current_user->username != $doc->owner->username) {
			$this->redirect('/error/denied/');
		}
		
		if ($this->arrayContains(array('user', 'perm', 'status'), $_POST)) {
			$usr = User::GetByName($_POST['user']);
			$perm = ACL::toPerm($_POST['perm']);
			$status = $_POST['status'];
			if ($usr && $perm) {
				$acl->SetPermission($usr, $doc, $perm, $status);
				$this->view->setMessage("Permissions updated", 'success');
				$this->redirect('/document/view/'.$id);
			} else {
				$this->view->setMessage('Error - User or permission not valid', 'error');
				$this->redirect('/document/changePermissions/'.$id);
			}
		} else {
			$this->view->setMessage('Error - Missing parameters', 'error');
			$this->redirect('/document/changePermissions/'.$id);
		}
	}
	
	public function setGroupPermission($id) {
		$doc = Document::GetById($id);
		$acl = new ACL();
		
		if (! $doc) {
			$this->redirect('/error/invalid/');
		}
		
		if (! $this->current_user->isAdministrator() &&
			  $this->current_user->username != $doc->owner->username) {
			$this->redirect('/error/denied/');
		}
		
		if ($this->arrayContains(array('group', 'perm', 'status'), $_POST)) {
			$group = Group::GetByName('user');
			$perm = ACL::toPerm($_POST['perm']);
			$status = $_POST['status'] == 'granted';
			if ($group && $perm) {
				$acl->SetPermission($group, $doc, $perm, $status);
				$this->redirect('/document/view/'.$id);
			} else {
				$this->view->load('set_group_permission');
				$this->view->render();
			}
		} else {
			$this->view->load('set_group_permission');
			$this->view->render();
		}
	}
}