-- Database initialization script for MySQL.

-- Static permission listing.  This enforces the valid values
-- for permissions in the UserPermissions and GroupPermissions.
CREATE TABLE Permissions (
	permission VARCHAR(10) NOT NULL PRIMARY KEY
) Engine=InnoDB;
INSERT INTO Permissions VALUES ('read'), ('edit'), ('create'), ('delete');

CREATE TABLE Users (
	username VARCHAR(255) NOT NULL,
	password CHAR(32) NOT NULL,
	PRIMARY KEY (username)
) Engine=InnoDB;

-- Add the default administrator account
INSERT INTO Users (username, password) VALUES ('admin', MD5('admin'));

CREATE TABLE Documents (
	id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
	title TEXT NOT NULL,
	content TEXT NOT NULL,
	owner VARCHAR(255),
	PRIMARY KEY (id),
	FOREIGN KEY (owner) REFERENCES Users(username)
		ON UPDATE CASCADE ON DELETE CASCADE
) Engine=InnoDB;

CREATE TABLE Groups (
	name VARCHAR(255) NOT NULL,
	description TEXT NOT NULL,
	PRIMARY KEY (name)
) Engine=InnoDB;
INSERT INTO Groups (name, description) VALUES ('Administrators', 'System administrators');

CREATE TABLE GroupMembers (
	group_name VARCHAR(255) NOT NULL,
	user_name VARCHAR(255) NOT NULL,
	PRIMARY KEY (group_name, user_name),
	FOREIGN KEY (user_name) REFERENCES Users(username)
		ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (group_name) REFERENCES Groups(name)
		ON UPDATE CASCADE ON DELETE CASCADE
) Engine=InnoDB;
INSERT INTO GroupMembers (group_name, user_name) VALUES ('Administrators', 'admin');

CREATE TABLE UserPermissions (
	document_id INTEGER UNSIGNED NOT NULL,
	username VARCHAR(255) NOT NULL,
	permission VARCHAR(10) NOT NULL,
	granted TINYINT(1),
	PRIMARY KEY (document_id, username, permission),
	FOREIGN KEY (document_id) REFERENCES Documents(id)
        ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (username) REFERENCES Users(username)
        ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (permission) REFERENCES Permissions(permission)
        ON UPDATE CASCADE ON DELETE CASCADE
) Engine=InnoDB;

CREATE TABLE GroupPermissions (
	document_id INTEGER UNSIGNED NOT NULL,
	group_name VARCHAR(255) NOT NULL,
	permission VARCHAR(10) NOT NULL,
	granted TINYINT(1),
	PRIMARY KEY (document_id, group_name, permission),
	FOREIGN KEY (document_id) REFERENCES Documents(id)
        ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (group_name) REFERENCES Groups(name)
        ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (permission) REFERENCES Permissions(permission)
        ON UPDATE CASCADE ON DELETE CASCADE
) Engine=InnoDB;
