<?php
class DB {
	
	protected $dbconn;
	
	public function __construct() {
		$this->dbconn = new PDO($GLOBALS['db']['dsn'],
								$GLOBALS['db']['username'],
								$GLOBALS['db']['password']);
		$this->dbconn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	}
	
	/**
	 * Convenience function to execute a select query and return the result set.
	 * @param string	$sql	The SQL query to execute.
	 * @param array 	$data	The parameters to execute the prepared query.
	 */
	protected function runSelect($sql, $data=array()) {
		//try {
			$query = $this->dbconn->prepare($sql);
			$query->execute($data);
			return $query->fetchAll();
		//} catch (PDOException $e) {
		//	error_log('Error on line '.__LINE__.' of '.__FILE__.': '.
		//			  $e->getMessage());
		//	return null;
		//}
	}
	
	/**
	 * Convenience function to fetch a single row.
	 * @param string $sql	The SQL query to execute.
	 * @param array $data	The data to execute the prepared query.
	 */
 	protected function runSingleSelect($sql, $data) {
		//try {
			$query = $this->dbconn->prepare($sql);
			$query->execute($data);
			$ret = $query->fetch();
			return $ret;
		//} catch (PDOException $e) {
		//	error_log('Error on line '.__LINE__.' of '.__FILE__.': '.
		//			  $e->getMessage());
		//	return null;
		//}
	}
	
	/**
	 * Convenience function to fetch a scalar value.
	 * @param string $sql	The SQL query to execute.
	 * @param mixed $data	The data in the single result column, or null for an empty result set.
	 */
	protected function runScalarSelect($sql, $data) {
		//try {
			$query = $this->dbconn->prepare($sql);
			$query->execute($data);
			$ret = $query->fetchColumn();
			return $ret === false ? null : $ret;
		//} catch (PDOException $e) {
		//	error_log('Error on line '.__LINE__.' of '.__FILE__.': '.
		//			  $e->getMessage());
		//	return null;
		//}
	}
	
	protected function runQuery($sql, $data) {
		//try {
			$query = $this->dbconn->prepare($sql);
	//var_dump($query); exit;
			return $query->execute($data);
		//} catch (PDOException $e) {
		//	error_log('Error on line '.__LINE__.' of '.__FILE__.': '.
		//			  $e->getMessage());
		//	return false;
		//}
	}
	
	public function SelectUser($username) {
		$sql = 'SELECT * FROM Users WHERE username = ?';
		return $this->runSingleSelect($sql, array($username));
	}
	
	public function UpdateUser($key, $data) {
		$sql = "UPDATE Users SET username = ?, password = ? WHERE username = ?";
		$params = array($data['username'], $data['password'], $key);
		return $this->runQuery($sql, $params);
	}
	
	public  function InsertUser($data) {
		$sql = "INSERT INTO Users (username, password) VALUES (?, ?)";
		$params = array($data['username'], $data['password']);
		return $this->runQuery($sql, $params);
	}
	
	public function DeleteUser($key) {
		$sql = "DELETE FROM Users WHERE username = ?";
		$params = array($key);
		return $this->runQuery($sql, $params);
	}
	
	public function SelectAllDocuments() {
		$sql = "SELECT * FROM Documents";
		return $this->runSelect($sql);
	}
	
	public function SelectDocument($id) {
		$sql = 'SELECT * FROM Documents WHERE id = ?';
		$params = array($id);
		return $this->runSingleSelect($sql, $params);
	}
	
	public function SelectDocumentByOwner($owner) {
		$sql = 'SELECT * FROM Documents WHERE owner = ?';
		$params = array($id);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectDocumentsByGroup($group) {
		$sql = 'SELECT * FROM Documents WHERE owner IN '.
		       '(SELECT user_name FROM GroupMembers WHERE group_name = ?)';
		$params = array($group);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectReadableDocuments($user) {
		$sql = 'SELECT * FROM Documents AS d WHERE EXISTS ' .
		       '(SELECT * FROM UserPermissions AS up WHERE '.
			   ' up.document_id = d.id AND up.username = ? AND '.
			   " up.pemission = 'read' and up.granted = 1) OR " .
			   '(SELECT * FROM GroupPermissions AS gp WHERE '.
			   ' gp.group_name IN (SELECT group_name FROM GroupMembers WHERE user = ?) AND '.
			   ' gp.document_id = d.id AND gp.group_name = ? AND '.
			   " gp.pemission = 'read' and gp.granted = 1)";
		$params = array($user, $user, $user);
		return $this->runSelect($sql, $params);
	}
	
	public function UpdateDocument($key, $data) {
		$sql = 'UPDATE Documents SET id = ?, title = ?, content = ?, owner = ? WHERE id = ?';
		$params = array($data['id'], $data['title'], $data['content'], $data['owner'], $data['id']);
		return $this->runQuery($sql, $params);
	}
	
	public function InsertDocument($data) {
		$sql = 'INSERT INTO Documents (title, content, owner) VALUES (?, ?, ?)';
		$params = array($data['title'], $data['content'], $data['owner']);
		# Need to add note to paper on handling auto-id.
		$ret = $this->runQuery($sql, $params);
		if ($ret) return $this->dbconn->lastInsertId();
		else return false;
	}
	
	public function DeleteDocument($key) {
		$sql = 'DELETE FROM Documents WHERE id = ?';
		$params = array($key);
		return $this->runQuery($sql, $params);
	}
	
	public function SelectAllGroups() {
		$sql = 'SELECT * FROM Groups';
		return $this->runSelect($sql);
	}
	
	public function SelectAllUsers() {
		$sql = 'SELECT * FROM Users';
		return $this->runSelect($sql);
	}
	
	public function SelectGroup($name) {
		$sql = 'SELECT * FROM Groups WHERE name = ?';
		$params = array($name);
		return $this->runSingleSelect($sql, $params);
	}
	
	public function InsertGroup($data) {
		$sql = 'INSERT INTO Groups (name, description) VALUES (?, ?)';
		$params = array($data['name'], $data['description']);
		return $this->runQuery($sql, $params);
	}
	
	public function UpdateGroup($key, $data) {
		$sql = 'UPDATE Group SET name = ?, description = ? WHERE name = ?';
		$params = array($data['name'], $data['description']);
		return $this->runQuery($sql, $params);
	}
	
	public function DeleteGroup($key) {
		$sql = 'DELETE FROM Groups WHERE name = ?';
		$params = array($key);
		return $this->runQuery($sql, $params);
	}
	
	public function SelectAdminMembershipByUser($user) {
		$sql = "SELECT CASE WHEN EXISTS ".
		       "(SELECT * FROM GroupMembers WHERE user_name = ? AND group_name = 'Administrators') THEN 1 ".
			   "ELSE 0 END";
		$params = array($user);
		return $this->runScalarSelect($sql, $params);
	}
	
	public function SelectGroupMembership($user, $group) {
		$sql = "SElECT CASE WHEN " .
		       "EXISTS (SELECT * FROM GroupMembers WHERE ".
			   "user_name = ? AND group_name = ?) THEN 1" .
			   "ELSE 0 END";
		$params = array($user, $group);
		return $this->runScalarSelect($sql, $params);
	}
	
	public function SelectGroupMembersByUser($user) {
		$sql = 'SELECT *
		        FROM GroupMembers AS gm JOIN Groups AS g ON gm.group_name = g.group_name
				WHERE user_name = ?';
		$params = array($user);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectGroupMembersByGroup($group) {
		$sql = 'SELECT *
		        FROM GroupMembers AS gm JOIN Users AS u ON gm.user_name = u.username
				WHERE group_name = ?';
		$params = array($group);
		return $this->runSelect($sql, $params);
	}
	
	public function InsertGroupMember($data) {
		$sql = 'INSERT INTO GroupMembers (group_name, user_name) VALUES (?, ?)';
		$params = array($data['group_name'], $data['user_name']);
		return $this->runQuery($sql, $params);
	}
	
	public function DeleteGroupMember($group, $user) {
		$sql = 'DELETE FROM Group WHERE group_name = ? AND user_name = ?';
		$params = array($data['group_name'], $data['user_name']);
		return $this->runQuery($sql, $params);
	}
	
	public function SelectUserPermissionByUser($user) {
		$sql = 'SELECT * FROM UserPermissions WHERE username = ?';
		$params = array($user);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectUserPermissionByDocument($id) {
		$sql = 'SELECT * FROM UserPermissions WHERE document_id = ?';
		$params = array($id);
		return $this->runSelect($sql, $params);
	}

	public function SelectUserPermission($user, $docid, $perm) {
		$sql = 'SELECT granted FROM UserPermissions WHERE ' .
		       'username = ? AND document_id = ? AND permission = ?';
		$params = array($user, $docid, $perm);
		return $this->runScalarSelect($sql, $params);
	}
	
	public function InsertUserPermission($data) {
		$sql = 'INSERT INTO UserPermissions '.
		       '(document_id, username, permission, granted) VALUES (?, ?, ?, ?)';
		$params = array($data['document_id'],
						$data['username'],
						$data['permission'],
						$data['granted']);
		return $this->runQuery($sql, $params);
	}
	
	public function DeleteUserPermission($docid, $user) {
		$sql = 'DELETE FROM UserPermissions '.
		       'WHERE document_id = ? AND username = ?';
		$params = array($docid, $user);
		return $this->runQuery($sql, $params);
	}
	
	public function UpdateUserPermission($user, $docid, $perm, $granted) {
		$sql = 'UPDATE UserPermissions SET granted = ? WHERE ' .
		       'username = ? AND document_id = ? AND permission = ?';
		$params = array($granted, $user, $docid, $perm);
		return $this->runQuery($sql, $params);
	}
	
	public function SelectGroupPermissionByUser($group) {
		$sql = 'SELECT * FROM GroupPermissions WHERE group_name = ?';
		$params = array($group);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectGroupPermissionByDocument($id) {
		$sql = 'SELECT * FROM GroupPermissions WHERE document_id = ?';
		$params = array($id);
		return $this->runSelect($sql, $params);
	}
	
	public function SelectGroupPermission($group, $docid, $perm) {
		$sql = 'SELECT * FROM GroupPermissions WHERE ' .
		       'group_name = ? AND document_id = ? AND permission = ?';
		$params = array($group, $docid, $perm);
		return $this->runScalarSelect($sql, $params);
	}
	
	public function InsertGroupPermission($data) {
		$sql = 'INSERT INTO GroupPermissions '.
		       '(document_id, group_name, permission, granted) VALUES (?, ?, ?, ?)';
		$params = array($data['document_id'],
						$data['group_name'],
						$data['permission'],
						$data['granted']);
		return $this->runQuery($sql, $params);
	}
	
	public function DeleteGroupPermission($docid, $group) {
		$sql = 'DELETE FROM GroupPermissions '.
		       'WHERE document_id = ? AND username = ?';
		$params = array($docid, $group);
		return $this->runQuery($sql, $params);
	}
	
	public function UpdateGroupPermission($group, $docid, $perm, $granted) {
		$sql = 'UPDATE GroupPermissions SET granted = ? WHERE ' .
		       'group_name = ? AND document_id = ? AND permission = ?';
		$params = array($granted, $group, $docid, $perm);
		return $this->runQuery($sql, $params);
	}
}